from .utils import EDITOR_PAGE

trust_test_code = """
import IPython.display
IPython.display.Javascript("window.javascriptExecuted = true")
"""


def test_trust(notebook_frontend):
    def save_notebook():
        notebook_frontend.evaluate("() => Jupyter.notebook.save_notebook()", page=EDITOR_PAGE)

    # Add a cell that executes javascript
    notebook_frontend.add_cell(index=0, content=trust_test_code)
    notebook_frontend.execute_cell(0)
    notebook_frontend.wait_for_cell_output(0)
    # Make sure the JavaScript executed
    assert notebook_frontend.evaluate("() => window.javascriptExecuted", page=EDITOR_PAGE) == True
    # Check that we see the 'Trusted' text on the page
    trusted = notebook_frontend.locate("#notification_trusted", page=EDITOR_PAGE)
    assert trusted.get_inner_text() == "Trusted"
    save_notebook()

    # refresh the page, should be trusted
    notebook_frontend.reload(EDITOR_PAGE)
    notebook_frontend.wait_for_kernel_ready()
    trusted = notebook_frontend.locate("#notification_trusted", page=EDITOR_PAGE)
    assert trusted.get_inner_text() == "Trusted"
    notebook_frontend.wait_for_cell_output(0)
    assert notebook_frontend.evaluate("() => window.javascriptExecuted", page=EDITOR_PAGE) == True



"""Test multiselect toggle"""


from .utils import EDITOR_PAGE


INITIAL_CELLS = ['print("a")', 'print("b")', 'print("c")']


def test_multiselect_toggle(prefill_notebook):
    notebook_frontend = prefill_notebook(INITIAL_CELLS)

    def extend_selection_by(delta):
        notebook_frontend.evaluate(
            f"Jupyter.notebook.extend_selection_by({delta});", EDITOR_PAGE)

    def n_selected_cells():
        return notebook_frontend.evaluate(
            "() => { return Jupyter.notebook.get_selected_cells().length; }", EDITOR_PAGE)

    def select_cells():
        notebook_frontend.focus_cell(0)
        extend_selection_by(2)

    # Test that cells, which start off not collapsed, are collapsed after
    # calling the multiselected cell toggle.
    select_cells()
    assert n_selected_cells() == 3
    notebook_frontend.evaluate("Jupyter.notebook.execute_selected_cells();", EDITOR_PAGE)
    select_cells()
    notebook_frontend.evaluate("Jupyter.notebook.toggle_cells_outputs();", EDITOR_PAGE)
    cell_output_states = notebook_frontend.evaluate(
        "() => { return Jupyter.notebook.get_cells().map(c => c.collapsed) }", EDITOR_PAGE)
    assert cell_output_states == [False] * 3, "ensure that all cells are not collapsed"

    # Test that cells, which start off not scrolled are scrolled after
    # calling the multiselected scroll toggle.
    select_cells()
    assert n_selected_cells() == 3
    notebook_frontend.evaluate("Jupyter.notebook.toggle_cells_outputs_scroll();", EDITOR_PAGE)
    cell_scrolled_states = notebook_frontend.evaluate(
        "() => { return Jupyter.notebook.get_cells().map(c => c.output_area.scroll_state) }", EDITOR_PAGE)
    assert all(cell_scrolled_states), "ensure that all have scrolling enabled"

    # Test that cells, which start off not cleared are cleared after
    # calling the multiselected scroll toggle.
    select_cells()
    assert n_selected_cells() == 3
    notebook_frontend.evaluate("Jupyter.notebook.clear_cells_outputs();", EDITOR_PAGE)
    cell_outputs_cleared = notebook_frontend.evaluate(
        "() => { return Jupyter.notebook.get_cells().map(c => c.output_area.element.html()) }", EDITOR_PAGE)
    assert cell_outputs_cleared == [""] * 3, "ensure that all cells are cleared"



"""Test save-as functionality"""


import traceback

from .utils import EDITOR_PAGE, EndToEndTimeout


def save_as(nb):
    JS = '() => Jupyter.notebook.save_notebook_as()'
    return nb.evaluate(JS, page=EDITOR_PAGE)


def get_notebook_name(nb):
    JS = '() => Jupyter.notebook.notebook_name'
    return nb.evaluate(JS, page=EDITOR_PAGE)


def set_notebook_name(nb, name):
    JS = f'() => Jupyter.notebook.rename("{name}")'
    nb.evaluate(JS, page=EDITOR_PAGE)


def test_save_as_nb(notebook_frontend):
    print('[Test] [test_save_readonly_as]')
    notebook_frontend.wait_for_kernel_ready()
    notebook_frontend.wait_for_selector(".input", page=EDITOR_PAGE)

    # Set a name for comparison later
    print('[Test] Set notebook name')
    set_notebook_name(notebook_frontend, name="nb1.ipynb")
    notebook_frontend.wait_for_condition(
        lambda: get_notebook_name(notebook_frontend) == 'nb1.ipynb',
        timeout=150,
        period=1
    )

    # Wait for Save As modal, save
    print('[Test] Save')
    save_as(notebook_frontend)

    # # Wait for modal to pop up
    print('[Test] Waiting for modal popup')
    notebook_frontend.wait_for_selector(".modal-footer", page=EDITOR_PAGE)
    dialog_element = notebook_frontend.locate(".modal-footer", page=EDITOR_PAGE)
    dialog_element.focus()

    print('[Test] Focus the notebook name input field, then click and modify its .value')
    notebook_frontend.wait_for_selector('.modal-body .form-control', page=EDITOR_PAGE)
    name_input_element = notebook_frontend.locate('.modal-body .form-control', page=EDITOR_PAGE)
    name_input_element.focus()
    name_input_element.click()
    notebook_name = 'new_notebook.ipynb'

    print('[Test] Begin attempts to fill the save dialog input and save the notebook')
    fill_attempts = 0

    def attempt_form_fill_and_save():
        # Application behavior here is HIGHLY variable, we use this for repeated attempts
        # ....................
        # This may be a retry, check if the application state reflects a successful save operation
        nonlocal fill_attempts
        if fill_attempts and get_notebook_name(notebook_frontend) == "new_notebook.ipynb":
            print('[Test]   Success from previous save attempt!')
            return True
        fill_attempts += 1
        print(f'[Test] Attempt form fill and save #{fill_attempts}')

        # Make sure the save prompt is visible
        if not name_input_element.is_visible():
            save_as(notebook_frontend)
            name_input_element.wait_for('visible')

        # Set the notebook name field in the save dialog
        print('[Test] Fill the input field')
        name_input_element.evaluate(f'(elem) => {{ elem.value = "new_notebook.ipynb"; return elem.value; }}')
        notebook_frontend.wait_for_condition(
            lambda: name_input_element.evaluate(
                f'(elem) => {{ elem.value = "new_notebook.ipynb"; return elem.value; }}') == 'new_notebook.ipynb',
            timeout=120,
            period=.25
        )
        # Show the input field value
        print('[Test] Name input field contents:')
        field_value = name_input_element.evaluate(f'(elem) => {{ return elem.value; }}')
        print('[Test]   ' + field_value)
        if field_value != 'new_notebook.ipynb':
            return False

        print('[Test] Locate and click the save button')
        save_element = dialog_element.locate('text=Save')
        save_element.wait_for('visible')
        save_element.focus()
        save_element.click()

        # Application lag may cause the save dialog to linger,
        # if it's visible wait for it to disappear before proceeding
        if save_element.is_visible():
            print('[Test] Save element still visible after save, wait for hidden')
            try:
                save_element.expect_not_to_be_visible(timeout=120)
            except EndToEndTimeout as err:
                traceback.print_exc()
                print('[Test]   Save button failed to hide...')

        # Check if the save operation succeeded (by checking notebook name change)
        notebook_frontend.wait_for_condition(
            lambda: get_notebook_name(notebook_frontend) == "new_notebook.ipynb", timeout=120, period=5
        )
        print(f'[Test] Notebook name: {get_notebook_name(notebook_frontend)}')
        print('[Test] Notebook name was changed!')
        return True

    # Retry until timeout (wait_for_condition retries upon func exception)
    notebook_frontend.wait_for_condition(attempt_form_fill_and_save, timeout=900, period=1)

    print('[Test] Check notebook name in URL')
    notebook_frontend.wait_for_condition(
        lambda: notebook_name in notebook_frontend.get_page_url(page=EDITOR_PAGE),
        timeout=120,
        period=5
    )



"""
store the current version info of nbclassic.

"""
import re

# Version string must appear intact for tbump versioning
__version__ = '1.1.0'

# Build up version_info tuple for backwards compatibility
pattern = r'(?P<major>\d+).(?P<minor>\d+).(?P<patch>\d+)(?P<rest>.*)'
match = re.match(pattern, __version__)
parts = [int(match[part]) for part in ['major', 'minor', 'patch']]
if match['rest']:
  parts.append(match['rest'])
version_info = tuple(parts)

# Downstream maintainer, when running `python.setup.py jsversion`,
# the version string is propagated to the JavaScript files,  do not forget to
# patch the JavaScript files in `.postN` release done by distributions.



"""Test kernel interrupt"""


from .utils import EDITOR_PAGE


def interrupt_from_menu(notebook_frontend):
    # Click interrupt button in kernel menu
    notebook_frontend.try_click_selector('#kernellink', page=EDITOR_PAGE)
    notebook_frontend.try_click_selector('#int_kernel', page=EDITOR_PAGE)
    notebook_frontend.press('Escape', page=EDITOR_PAGE)


def interrupt_from_keyboard(notebook_frontend):
    notebook_frontend.type("ii", page=EDITOR_PAGE)
    notebook_frontend.press('Escape', page=EDITOR_PAGE)


def test_interrupt(notebook_frontend):
    """ Test the interrupt function using both the button in the Kernel menu and the keyboard shortcut "ii"
    """

    text = ('import time\n'
            'for x in range(3):\n'
            '   time.sleep(1)')

    notebook_frontend.edit_cell(index=0, content=text)

    for interrupt_method in (interrupt_from_menu, interrupt_from_keyboard):
        notebook_frontend.clear_cell_output(0)
        notebook_frontend.to_command_mode()
        notebook_frontend.execute_cell(0)

        interrupt_method(notebook_frontend)

        # Wait for an output to appear
        output = notebook_frontend.wait_for_cell_output(0)
        assert 'KeyboardInterrupt' in output.get_inner_text()
